unit Unit1;

interface

uses
  SysUtils, Forms, Controls, Classes,
  Menus, ExtCtrls, ImgList, ComCtrls,
  Dialogs, RawRegistry, RegUtils, ShellAPI;

type
  TfmMain = class(TForm)
    SB: TStatusBar;
    TV: TTreeView;
    Splitter1: TSplitter;
    LV: TListView;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    N1: TMenuItem;
    Open1: TMenuItem;
    ImageList1: TImageList;
    dlgOpen: TOpenDialog;
    ImageList2: TImageList;
    N2: TMenuItem;
    procedure Open1Click(Sender: TObject);
    procedure TVMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TVExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure TVChange(Sender: TObject; Node: TTreeNode);
    procedure Exit1Click(Sender: TObject);
    procedure N2Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;
  R    : TRawRegistry;

procedure AddKeysToNode(const Node: TTreeNode; const Path: String);
function GetCurrentKeyname(const N: TTreeNode): String;
procedure ShowValues;
procedure ShowChildren(const N: TTreeNode);
procedure ShowSharedPasswords;

implementation

{$R *.DFM}

procedure AddKeysToNode(const Node: TTreeNode; const Path: String);
var
  S: TStringList;
  I: Integer;
  N: TTreeNode;
begin
  try
  if R.OpenKey(Path) then
    with fmMain.TV, R do
    begin
      S := TStringList.Create;
      S.Sorted := True;
      GetKeyNames(S);
      Enabled := False;
      for I := 0 to S.Count - 1 do
      begin
       N := Items.AddChild(Node,S.Strings[I]);
       N.HasChildren := True;
       N.SelectedIndex := 1;
{       if OpenKey(Path + '\' + S.Strings[I]) then
         if HasSubKeys then N.HasChildren := True
         else N.HasChildren := False; } //Costs too much ;) [+] sign verificator
      end;
      S.Destroy;
      Enabled := True;
    end;
  except
  end;
end;

function GetCurrentKeyname(const N: TTreeNode): String;
var
  F: TTreeNode;
  S: String;
begin
  S := '\' + N.Text;
  F := N.Parent;
  while F <> nil do
  begin
    S := '\' + F.Text + S;
    F := F.Parent;
  end;
  Result := S;
end;

procedure ShowValues;
var
  S: TStringList;
  I: Integer;
  F: TListItem;
  V: TRegDataInfo;
  D: array [0..4096] of Byte;
begin
  try
  Screen.Cursor := crHourglass;
  with fmMain.LV, R, S, F do
  begin
    Enabled := False;
    S := TStringList.Create;
    Sorted := True;
    GetValueNames(S);
    Items.Clear;
    for I := 0 to Count - 1 do
    begin
      F := Items.Add;
      Caption := Strings[I];
      GetDataInfo(Strings[I],V);
      case V.RegData of
        rdString :
          begin
            F.ImageIndex := 1;
            SubItems.Add('"' + ReadString(Strings[I]) + '"');
          end;
        rdInteger: SubItems.Add(DwordToStr(ReadInteger(Strings[I])));
        rdBinary :
          begin
            if V.DataSize < 4096 then
            begin
              ReadBinaryData(Strings[I],D,V.DataSize);
              SubItems.Add(BinaryToBytes32(D,V.DataSize));  { only 32 bytes to show }
            end
            else
            begin
              ReadBinaryData(Strings[I],D,32);
              SubItems.Add(BinaryToBytes32(D,32));  { only 32 bytes to show }
            end
          end
      end
    end;
    Enabled := True;
  end;
  finally
  Screen.Cursor := crDefault;
  end;
end;

procedure ShowChildren(const N: TTreeNode);
var
  KeyName: String;
begin
  try
  Screen.Cursor := crHourglass;
  with fmMain do
  if N <> nil then
    begin
      KeyName := GetCurrentKeyname(N);
      if N.GetFirstChild = nil then AddKeysToNode(N,KeyName);
    end;
  finally
  Screen.Cursor := crDefault;
  end;
end;

procedure ShowNode(const Node: TTreeNode);
var
  Path: String;
begin
  try
  if (Node <> nil) then
  begin
    Path := GetCurrentKeyName(Node);
    if Path = R.CurrentPath then Exit;
    R.OpenKey(Path);
    if not(R.HasSubKeys) then Node.HasChildren := False;
    ShowValues;
    fmMain.SB.SimpleText := R.CurrentPath;
  end;
  except
  end;
end;

procedure ShowSharedPasswords;
var
  S: TStringList;
  I: Integer;
  L1,L2: Integer;
  P1,P2: String;
  G,W: array [1..8] of Byte;
  N: TListItem;
const
  PP = '\Software\Microsoft\Windows\CurrentVersion\Network\LanMan';

begin
  try
  with R do
  if OpenKey(PP) then
  begin
    S := TStringList.Create;
    GetKeyNames(S);
    for I := 0 to S.Count - 1 do
      if OpenKey(PP + '\' + S.Strings[I]) then
      with fmMain.LV do
      begin
        L1 := GetDataSize('Parm1enc');
        if L1 > 0 then
        begin
          ReadBinaryData('Parm1enc',G,L1);
          P1 := DecodeSharePassword(G,L1);
        end;
        L2 := GetDataSize('Parm2enc');
        if L2 > 0 then
        begin
          ReadBinaryData('Parm2enc',W,L2);
          P2 := DecodeSharePassword(W,L2)
        end;
        N := Items.Add;
        N.Caption := 'Shared ' + ReadString('Path');
        N.SubItems.Add('Password(s): ' + P1 + ' ' + P2);
      end;
  end;
  except
  end;
end;

procedure TfmMain.Open1Click(Sender: TObject);
begin
  try
  if dlgOpen.Execute then
  begin
    if R <> nil then
    begin
      LV.Enabled := False;
       LV.Items.Clear;
      LV.Enabled := True;

      TV.Enabled := False;
       TV.Items.Clear;
      TV.Enabled := True;

      R.Destroy;
    end;
    TV.HideSelection := False;
    R := TRawRegistry.Create(dlgOpen.FileName);
    AddKeysToNode(nil,'\');
    ShowSharedPasswords;
    R.OpenKey('\');
    SB.SimpleText := dlgOpen.FileName;
  end;
  except
    on E: ERegistryException do SB.SimpleText := E.Message;
  end;
end;

procedure TfmMain.TVExpanding(Sender: TObject; Node: TTreeNode;
  var AllowExpansion: Boolean);
var
  P: String;
begin
  P := R.CurrentPath;
  if Node.GetFirstChild = nil then ShowChildren(Node);
  R.OpenKey(P);
end;

procedure TfmMain.TVMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  N : TTreeNode;
begin
  N := TV.GetNodeAt(X,Y);
  ShowNode(N);
end;

procedure TfmMain.TVChange(Sender: TObject; Node: TTreeNode);
begin
  ShowNode(Node);
end;

procedure TfmMain.Exit1Click(Sender: TObject);
begin
  R.Free;
  Application.Terminate;
end;

procedure TfmMain.N2Click(Sender: TObject);
begin
  with Application do
  ShellAbout(Handle,PChar(Title),PChar('Ilya Malyarenko [malyarenko@pisem.net]'),Icon.Handle);
end;

end.
